package net.wanji.opt.servicev2.implv2;

import lombok.extern.slf4j.Slf4j;
import net.wanji.opt.dao.mapper.TrunkLineMapper;
import net.wanji.opt.servicev2.TrunkLineService;
import net.wanji.opt.synthesis.pojo.TrunkLineCrossProblemEntity;
import net.wanji.opt.synthesis.pojo.TrunkLineProblemDescribeEntity;
import net.wanji.opt.vo2.CrossRealTimeAlarmVO;
import net.wanji.opt.vo2.TrunkLineCrossProblemVO;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.*;

@Service
@Slf4j
public class TrunkLineImpl implements TrunkLineService {
    @Resource
    private TrunkLineMapper trunkLineMapper;

    @Override
    public List<TrunkLineProblemDescribeEntity> getTrunkLineProblemDescribe(String greenID , String time) {
        return trunkLineMapper.getTrunkLineProblemDescribe(greenID,time);
    }

    @Override
    public List<CrossRealTimeAlarmVO> getTrunkLineRealTimeAlarm(Integer greenID, String time) {
        return trunkLineMapper.getTrunkLineRealTimeAlarm(greenID, time);
    }

    @Override
    public List<TrunkLineCrossProblemVO> getTrunkLineCrossProblem(Integer greenID, String time) {
        List<TrunkLineCrossProblemEntity> list= trunkLineMapper.getTrunkLineCrossProblem(greenID, time);
        TrunkLineCrossProblemEntity ioDir = trunkLineMapper.getIODir(greenID);
        List<TrunkLineCrossProblemVO> voList = new ArrayList<>();
        Map<String, String> IOdirMap = getDir(ioDir.getInDir(), ioDir.getOutDir());

        for (TrunkLineCrossProblemEntity e : list) {
            TrunkLineCrossProblemVO vo = new TrunkLineCrossProblemVO();

            vo.setCrossID(e.getCrossID());
            vo.setName(e.getName());

            if (e.getDirection() != null) {
                Map<String, Integer> directionMap = new HashMap<>();
                for (String s : e.getDirection()) {
                    directionMap.put(s, directionMap.getOrDefault(s, 0) + 1);
                }

                Integer count = directionMap.getOrDefault(IOdirMap.get("dir"), 0);

                if (count == 0) { // 不匹配
                    vo.setStatus_1(0);
                    vo.setStatus_2(0);
                    vo.setStatus_1_name("正常");
                    vo.setStatus_2_name("正常");
                    vo.setDir_1_name(IOdirMap.get("dirName"));
                    vo.setDir_2_name(IOdirMap.get("dirNameReverse"));
                } else if (count == 1) { // 匹配一项
                    vo.setStatus_1(e.getStatus());
                    vo.setStatus_2(0);
                    vo.setStatus_2_name(getStatusName(e.getStatus()));
                    vo.setStatus_1_name("正常");
                    vo.setDir_1_name(IOdirMap.get("dirName"));
                    vo.setDir_2_name(IOdirMap.get("dirNameReverse"));
                } else if (count == 2) { // 匹配两项
                    vo.setStatus_1(e.getStatus());
                    vo.setStatus_2(e.getStatus());
                    vo.setStatus_1_name(getStatusName(e.getStatus()));
                    vo.setStatus_2_name(getStatusName(e.getStatus()));
                    vo.setDir_1_name(IOdirMap.get("dirName"));
                    vo.setDir_2_name(IOdirMap.get("dirNameReverse"));
                }
                voList.add(vo);
            } else {
                vo.setStatus_1(0);
                vo.setStatus_2(0);
                vo.setStatus_1_name("正常");
                vo.setStatus_2_name("正常");
                vo.setDir_1_name(IOdirMap.get("dirName"));
                vo.setDir_2_name(IOdirMap.get("dirNameReverse"));

                voList.add(vo);
            }
        }

        return voList;
    }

    private Map<String, String> getDir(Integer inDir, Integer outDir) {
        // 1 北 3 东 5 南 7 西
        // 0 北向南 2东向西 4 西向东 6 南向北
        Map<String, String> result = new HashMap<>();

        if (inDir == 5 && outDir == 1) { // 南向北
            result.put("dir", "6");
            result.put("dirName", "南向北");
            result.put("dirNameReverse", "北向南");
            return result;
        } else if (inDir == 7 && outDir == 3) { // 西向东
            result.put("dir", "4");
            result.put("dirName", "西向东");
            result.put("dirNameReverse", "东向西");
            return result;
        } else if (inDir == 3 && outDir == 7) { // 东向西
            result.put("dir", "2");
            result.put("dirName", "东向西");
            result.put("dirNameReverse", "西向东");
            return result;
        } else if (inDir == 1 && outDir == 5) { // 北向南
            result.put("dir", "0");
            result.put("dirName", "北向南");
            result.put("dirNameReverse", "南向北");
            return result;
        }
        return Collections.emptyMap();
    }

    private String getStatusName(Integer status) {
        switch(status) {
            case -1:
            case 0:
                return "正常";
            case 1:
                return "失衡";
            case 3:
                return "拥堵";
            case 4:
                return "死锁";
            case 5:
                return "相位空放";
        }
        return "";
    }
}
